"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_1 = __importDefault(require("../evaluator/configuration"));
exports.BOOLEAN_OBJ = 'BOOLEAN', exports.BUILTIN_OBJ = 'BUILTIN', exports.ERROR_OBJ = 'ERROR', exports.COMMENT_OBJ = 'COMMENT', exports.FUNCTION_OBJ = 'FUNCTION', exports.INTEGER_OBJ = 'INTEGER', exports.FLOAT_OBJ = 'FLOAT', exports.NULL_OBJ = 'NULL', exports.RETURN_VALUE_OBJ = 'RETURN_VALUE', exports.STRING_OBJ = 'STRING', exports.ARRAY_OBJ = 'ARRAY', exports.HASH_OBJ = 'HASH';
class HashKey {
    constructor(type, value) {
        this.Type = type;
        this.Value = value;
        this.Match = this.Type + this.Value;
    }
}
exports.HashKey = HashKey;
class OBoolean {
    constructor(value) {
        this.Value = value;
    }
    Type() {
        return exports.BOOLEAN_OBJ;
    }
    Inspect() {
        return this.Value ? 'true' : 'false';
    }
    toString() {
        return this.Value ? 'true' : 'false';
    }
    toValue() {
        return this.Value ? true : false;
    }
    HashKey() {
        let value;
        if (this.Value) {
            value = 1;
        }
        else {
            value = 0;
        }
        return new HashKey(this.Type(), value);
    }
}
exports.OBoolean = OBoolean;
class OInteger {
    constructor(value) {
        this.Value = value;
    }
    Type() {
        return exports.INTEGER_OBJ;
    }
    Inspect() {
        return String(this.Value);
    }
    toString() {
        return String(this.Value);
    }
    toValue() {
        return this.Value;
    }
    HashKey() {
        return new HashKey(this.Type(), this.Value);
    }
}
exports.OInteger = OInteger;
class OFloat {
    constructor(value) {
        this.Value = value;
    }
    Type() {
        return exports.FLOAT_OBJ;
    }
    Inspect() {
        return String(this.Value);
    }
    toString() {
        return String(this.Value);
    }
    toValue() {
        return this.Value;
    }
    HashKey() {
        return new HashKey(this.Type(), this.Value);
    }
}
exports.OFloat = OFloat;
class ONull {
    Type() {
        return exports.NULL_OBJ;
    }
    Inspect() {
        if (configuration_1.default.outputNULL)
            return 'null';
        return '';
    }
    toString() {
        return 'null';
    }
    toValue() {
        return null;
    }
}
exports.ONull = ONull;
class ReturnValue {
    constructor(value) {
        this.Value = value;
    }
    Type() {
        return exports.RETURN_VALUE_OBJ;
    }
    Inspect() {
        return this.Value.Inspect();
    }
    toString() {
        return this.Value.Inspect();
    }
    toValue() {
        return this.Value.toValue();
    }
}
exports.ReturnValue = ReturnValue;
class OError {
    constructor(message) {
        this.Message = message;
    }
    Type() {
        return exports.ERROR_OBJ;
    }
    Inspect() {
        return `Error: ${this.Message}`;
    }
    toString() {
        return `"Error: ${this.Message}"`;
    }
    toValue() {
        return new Error(this.Message);
    }
}
exports.OError = OError;
class OFunction {
    constructor(parameters, body, env) {
        this.Parameters = parameters;
        this.Body = body;
        this.Env = env;
    }
    Type() {
        return exports.FUNCTION_OBJ;
    }
    Inspect() {
        if (!configuration_1.default.outputFunctionBody)
            return 'fn';
        let params = this.Parameters.map(p => p.String());
        return `fn(${params.join(', ')}) {\n  ${this.Body.String()}\n}`;
    }
    toString() {
        return `fn`;
    }
    toValue() {
        return new Error('OFunction values not implemented');
    }
}
exports.OFunction = OFunction;
function hashCode(str) {
    return Array.from(str).reduce((s, c) => (Math.imul(31, s) + c.charCodeAt(0)) | 0, 0);
}
class OString {
    constructor(value) {
        this.Value = value;
    }
    Type() {
        return exports.STRING_OBJ;
    }
    Inspect() {
        return this.Value;
    }
    toString() {
        return `"${this.Value}"`;
    }
    toValue() {
        return this.Value;
    }
    HashKey() {
        let value = hashCode(this.Value);
        return new HashKey(this.Type(), value);
    }
}
exports.OString = OString;
class OComment {
    constructor(message) {
        this.Message = message;
    }
    Type() {
        return exports.STRING_OBJ;
    }
    Inspect() {
        return `# ${this.Message}`;
    }
    toString() {
        return '';
    }
    toValue() {
        return this.Message;
    }
    HashKey() {
        let message = hashCode(this.Message);
        return new HashKey(this.Type(), message);
    }
}
exports.OComment = OComment;
class Builtin {
    constructor(fn) {
        this.Fn = fn;
    }
    Type() {
        return exports.BUILTIN_OBJ;
    }
    Inspect() {
        return 'builtin function';
    }
    toString() {
        return '"builtin function"';
    }
    toValue() {
        return new Error('Builtin values not implemented');
    }
}
exports.Builtin = Builtin;
class OArray {
    constructor(elements) {
        this.Elements = elements;
    }
    Type() {
        return exports.ARRAY_OBJ;
    }
    Inspect() {
        let elements = this.Elements.map(e => (e ? e.Inspect() : null));
        return `[${elements.join(', ')}]`;
    }
    toString() {
        let elements = this.Elements.map(e => (e ? e.toString() : null));
        return `[${elements.join(', ')}]`;
    }
    toValue() {
        let elements = this.Elements.map(e => (e ? e.toValue() : null));
        return elements;
    }
}
exports.OArray = OArray;
class HashPair {
    constructor(key, value) {
        this.Key = key;
        this.Value = value;
    }
}
exports.HashPair = HashPair;
class OHash {
    constructor(pairs) {
        this.Pairs = new Map();
        this.Pairs = pairs;
    }
    Type() {
        return exports.HASH_OBJ;
    }
    Inspect() {
        let pairs = [];
        this.Pairs.forEach(v => pairs.push(`${v.Key.Inspect()}:${v.Value.Inspect()}`));
        return `{${pairs.join(', ')}}`;
    }
    toString() {
        let pairs = [];
        this.Pairs.forEach(v => pairs.push(`${v.Key.toString()}: ${v.Value.toString()}`));
        return `{${pairs.join(', ')}}`;
    }
    toValue() {
        let obj = {};
        this.Pairs.forEach(v => (obj[v.Key.toString()] = v.Value.toString()));
        return obj;
    }
}
exports.OHash = OHash;
//# sourceMappingURL=object.js.map