"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const token_1 = require("../token/token");
const ast_1 = require("../ast/ast");
exports.LOWEST = 1, exports.LOR = 2, exports.LAND = 3, exports.EQUALS = 4, exports.LESSGREATER = 5, exports.SUM = 6, exports.PRODUCT = 7, exports.PREFIX = 8, exports.CALL = 9, exports.INDEX = 10;
exports.precedences = {
    [token_1.TokenType.LOR]: exports.EQUALS,
    [token_1.TokenType.LAND]: exports.EQUALS,
    [token_1.TokenType.EQ]: exports.EQUALS,
    [token_1.TokenType.NOT_EQ]: exports.EQUALS,
    [token_1.TokenType.LT]: exports.LESSGREATER,
    [token_1.TokenType.LTE]: exports.LESSGREATER,
    [token_1.TokenType.GT]: exports.LESSGREATER,
    [token_1.TokenType.GTE]: exports.LESSGREATER,
    [token_1.TokenType.PLUS]: exports.SUM,
    [token_1.TokenType.MINUS]: exports.SUM,
    [token_1.TokenType.SLASH]: exports.PRODUCT,
    [token_1.TokenType.ASTERISK]: exports.PRODUCT,
    [token_1.TokenType.EXPONENT]: exports.PRODUCT,
    [token_1.TokenType.REM]: exports.PRODUCT,
    [token_1.TokenType.LPAREN]: exports.CALL,
    [token_1.TokenType.LBRACKET]: exports.INDEX,
};
class Parser {
    constructor(lexer) {
        this.errors = [];
        this.prefixParseFns = {};
        this.infixParseFns = {};
        this.l = lexer;
        this.registerPrefix(token_1.TokenType.COMMENT, this.parseComment.bind(this));
        this.registerPrefix(token_1.TokenType.IDENT, this.parseIdentifier.bind(this));
        this.registerPrefix(token_1.TokenType.INT, this.parseIntegerLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.FLOAT, this.parseFloatLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.STRING, this.parseStringLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.BANG, this.parsePrefixExpression.bind(this));
        this.registerPrefix(token_1.TokenType.MINUS, this.parsePrefixExpression.bind(this));
        this.registerPrefix(token_1.TokenType.TRUE, this.parseBoolean.bind(this));
        this.registerPrefix(token_1.TokenType.FALSE, this.parseBoolean.bind(this));
        this.registerPrefix(token_1.TokenType.LPAREN, this.parseGroupedExpression.bind(this));
        this.registerPrefix(token_1.TokenType.IF, this.parseIfExpression.bind(this));
        this.registerPrefix(token_1.TokenType.FUNCTION, this.parseFunctionLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.WHILE, this.parseWhileLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.LBRACKET, this.parseArrayLiteral.bind(this));
        this.registerPrefix(token_1.TokenType.LBRACE, this.parseHashLiteral.bind(this));
        [
            token_1.TokenType.PLUS,
            token_1.TokenType.MINUS,
            token_1.TokenType.SLASH,
            token_1.TokenType.ASTERISK,
            token_1.TokenType.EXPONENT,
            token_1.TokenType.EQ,
            token_1.TokenType.NOT_EQ,
            token_1.TokenType.LT,
            token_1.TokenType.LTE,
            token_1.TokenType.GT,
            token_1.TokenType.GTE,
            token_1.TokenType.REM,
            token_1.TokenType.LAND,
            token_1.TokenType.LOR,
        ].forEach(value => this.registerInfix(value, this.parseInfixExpression.bind(this)));
        this.registerInfix(token_1.TokenType.LPAREN, this.parseCallExpression.bind(this));
        this.registerInfix(token_1.TokenType.LBRACKET, this.parseIndexExpression.bind(this));
        this.peekToken = this.l.NextToken();
        this.curToken = this.peekToken;
        this.peekToken = this.l.NextToken();
    }
    Errors() {
        return this.errors;
    }
    nextToken() {
        this.curToken = this.peekToken;
        this.peekToken = this.l.NextToken();
    }
    ParseProgram() {
        let program = new ast_1.ASTProgram();
        while (!this.curTokenIs(token_1.TokenType.EOF)) {
            let stmt = this.parseStatement();
            if (stmt !== null) {
                program.Statements.push(stmt);
            }
            this.nextToken();
        }
        return program;
    }
    parseStatement() {
        switch (this.curToken.Type) {
            case token_1.TokenType.LET:
                return this.parseLetStatement();
            case token_1.TokenType.RETURN:
                return this.parseReturnStatement();
            default:
                return this.parseExpressionStatement();
        }
    }
    parseLetStatement() {
        let curToken = this.curToken;
        if (!this.expectPeek(token_1.TokenType.IDENT)) {
            return null;
        }
        let Name = new ast_1.Identifier(this.curToken, this.curToken.Literal);
        if (!this.expectPeek(token_1.TokenType.ASSIGN)) {
            return null;
        }
        this.nextToken();
        let Value = this.parseExpression(exports.LOWEST);
        if (this.peekTokenIs(token_1.TokenType.SEMICOLON)) {
            this.nextToken();
        }
        return new ast_1.LetStatement(curToken, Name, Value);
    }
    parseReturnStatement() {
        let stmt = new ast_1.ReturnStatement(this.curToken);
        this.nextToken();
        stmt.ReturnValue = this.parseExpression(exports.LOWEST);
        if (this.peekTokenIs(token_1.TokenType.SEMICOLON)) {
            this.nextToken();
        }
        return stmt;
    }
    parseExpressionStatement() {
        let curToken = this.curToken;
        let Expression = this.parseExpression(exports.LOWEST);
        if (this.peekTokenIs(token_1.TokenType.SEMICOLON)) {
            this.nextToken();
        }
        return new ast_1.ExpressionStatement(curToken, Expression);
    }
    parseExpression(precedence) {
        if (!this.prefixParseFns[this.curToken.Type]) {
            this.noPrefixParseFnError(this.curToken);
            return null;
        }
        let prefix = this.prefixParseFns[this.curToken.Type];
        let leftExp = prefix();
        while (!this.peekTokenIs(token_1.TokenType.SEMICOLON) && precedence < this.peekPrecedence()) {
            if (!this.infixParseFns[this.peekToken.Type]) {
                return leftExp;
            }
            let infix = this.infixParseFns[this.peekToken.Type];
            this.nextToken();
            leftExp = infix(leftExp);
        }
        return leftExp;
    }
    parseIdentifier() {
        return new ast_1.Identifier(this.curToken, this.curToken.Literal);
    }
    parseComment() {
        return new ast_1.Comment(this.curToken, this.curToken.Literal);
    }
    formatError(t, msg) {
        return `${msg} at ${t.Position.String()}`;
    }
    parseIntegerLiteral() {
        try {
            let value = parseInt(this.curToken.Literal, 10);
            let lit = new ast_1.IntegerLiteral(this.curToken, value);
            return lit;
        }
        catch {
            let msg = this.formatError(this.curToken, `could not parse ${this.curToken.Literal} as integer`);
            this.errors.push(msg);
            return null;
        }
    }
    parseFloatLiteral() {
        try {
            let value = parseFloat(this.curToken.Literal);
            let lit = new ast_1.FloatLiteral(this.curToken, value);
            return lit;
        }
        catch {
            let msg = this.formatError(this.curToken, `could not parse ${this.curToken.Literal} as float`);
            this.errors.push(msg);
            return null;
        }
    }
    parseStringLiteral() {
        return new ast_1.StringLiteral(this.curToken, this.curToken.Literal);
    }
    curTokenIs(t) {
        return this.curToken.Type === t;
    }
    peekTokenIs(t) {
        return this.peekToken.Type === t;
    }
    expectPeek(t) {
        if (this.peekTokenIs(t)) {
            this.nextToken();
            return true;
        }
        else {
            this.peekError(t);
            return false;
        }
    }
    peekError(t) {
        if (this.peekToken === null)
            return;
        let msg = this.formatError(this.peekToken, `expected next token to be ${t}, got ${this.peekToken.Type} instead`);
        this.errors.push(msg);
    }
    registerPrefix(tokenType, fn) {
        this.prefixParseFns[tokenType] = fn;
    }
    registerInfix(tokenType, fn) {
        this.infixParseFns[tokenType] = fn;
    }
    noPrefixParseFnError(t) {
        let msg = this.formatError(t, `no prefix parse function for ${t.Type} found: ${t.Literal}`);
        this.errors.push(msg);
    }
    parsePrefixExpression() {
        let curToken = this.curToken;
        let curLiteral = this.curToken.Literal;
        this.nextToken();
        return new ast_1.PrefixExpression(curToken, curLiteral, this.parseExpression(exports.PREFIX));
    }
    peekPrecedence() {
        if (this.peekToken === null)
            return exports.LOWEST;
        if (exports.precedences[this.peekToken.Type])
            return exports.precedences[this.peekToken.Type];
        return exports.LOWEST;
    }
    curPrecedence() {
        if (exports.precedences[this.curToken.Type])
            return exports.precedences[this.curToken.Type];
        return exports.LOWEST;
    }
    parseInfixExpression(left) {
        let curToken = this.curToken;
        let curLiteral = this.curToken.Literal;
        let precedence = this.curPrecedence();
        this.nextToken();
        return new ast_1.InfixExpression(curToken, left, curLiteral, this.parseExpression(precedence));
    }
    parseBoolean() {
        return new ast_1.AstBoolean(this.curToken, this.curTokenIs(token_1.TokenType.TRUE));
    }
    parseGroupedExpression() {
        this.nextToken();
        let exp = this.parseExpression(exports.LOWEST);
        if (!this.expectPeek(token_1.TokenType.RPAREN)) {
            return null;
        }
        return exp;
    }
    parseIfExpression() {
        let curToken = this.curToken;
        if (!this.expectPeek(token_1.TokenType.LPAREN)) {
            return null;
        }
        this.nextToken();
        let Condition = this.parseExpression(exports.LOWEST);
        if (!this.expectPeek(token_1.TokenType.RPAREN)) {
            return null;
        }
        if (!this.expectPeek(token_1.TokenType.LBRACE)) {
            return null;
        }
        let Consequence = this.parseBlockStatement();
        let Alternative = null;
        if (this.peekTokenIs(token_1.TokenType.ELSE)) {
            this.nextToken();
            if (!this.expectPeek(token_1.TokenType.LBRACE)) {
                return null;
            }
            Alternative = this.parseBlockStatement();
        }
        return new ast_1.IfExpression(curToken, Condition, Consequence, Alternative);
    }
    parseBlockStatement() {
        let block = new ast_1.BlockStatement(this.curToken);
        this.nextToken();
        while (!this.curTokenIs(token_1.TokenType.RBRACE) && !this.curTokenIs(token_1.TokenType.EOF)) {
            let stmt = this.parseStatement();
            if (stmt !== null) {
                block.Statements.push(stmt);
            }
            this.nextToken();
        }
        return block;
    }
    parseFunctionLiteral() {
        let curToken = this.curToken;
        if (!this.expectPeek(token_1.TokenType.LPAREN)) {
            return null;
        }
        let Parameters = this.parseFunctionParameters();
        if (!this.expectPeek(token_1.TokenType.LBRACE)) {
            return null;
        }
        let Body = this.parseBlockStatement();
        return new ast_1.FunctionLiteral(curToken, Parameters, Body);
    }
    parseWhileLiteral() {
        let curToken = this.curToken;
        if (!this.expectPeek(token_1.TokenType.LPAREN)) {
            return null;
        }
        let Expression = this.parseExpression(exports.LOWEST);
        if (!this.expectPeek(token_1.TokenType.LBRACE)) {
            return null;
        }
        let Body = this.parseBlockStatement();
        return new ast_1.WhileLiteral(this.curToken, Expression, Body);
    }
    parseFunctionParameters() {
        let identifiers = [];
        if (this.peekTokenIs(token_1.TokenType.RPAREN)) {
            this.nextToken();
            return identifiers;
        }
        this.nextToken();
        let ident = new ast_1.Identifier(this.curToken, this.curToken.Literal);
        identifiers.push(ident);
        while (this.peekTokenIs(token_1.TokenType.COMMA)) {
            this.nextToken();
            this.nextToken();
            ident = new ast_1.Identifier(this.curToken, this.curToken.Literal);
            identifiers.push(ident);
        }
        if (!this.expectPeek(token_1.TokenType.RPAREN)) {
            return [];
        }
        return identifiers;
    }
    parseCallExpression(func) {
        return new ast_1.CallExpression(this.curToken, func, this.parseExpressionList(token_1.TokenType.RPAREN));
    }
    parseArrayLiteral() {
        return new ast_1.ArrayLiteral(this.curToken, this.parseExpressionList(token_1.TokenType.RBRACKET));
    }
    parseExpressionList(end) {
        let list = [];
        if (this.peekTokenIs(end)) {
            this.nextToken();
            return list;
        }
        this.nextToken();
        list.push(this.parseExpression(exports.LOWEST));
        while (this.peekTokenIs(token_1.TokenType.COMMA)) {
            this.nextToken();
            this.nextToken();
            list.push(this.parseExpression(exports.LOWEST));
        }
        if (!this.expectPeek(end)) {
            return [];
        }
        return list;
    }
    parseIndexExpression(left) {
        let curToken = this.curToken;
        this.nextToken();
        let exp = new ast_1.IndexExpression(curToken, left, this.parseExpression(exports.LOWEST));
        if (!this.expectPeek(token_1.TokenType.RBRACKET)) {
            return null;
        }
        return exp;
    }
    parseHashLiteral() {
        let hash = new ast_1.HashLiteral(this.curToken, new Map());
        while (!this.peekTokenIs(token_1.TokenType.RBRACE)) {
            this.nextToken();
            let key = this.parseExpression(exports.LOWEST);
            if (!this.expectPeek(token_1.TokenType.COLON)) {
                return null;
            }
            this.nextToken();
            let value = this.parseExpression(exports.LOWEST);
            hash.Pairs.set(key, value);
            if (!this.peekTokenIs(token_1.TokenType.RBRACE) && !this.expectPeek(token_1.TokenType.COMMA)) {
                return null;
            }
        }
        if (!this.expectPeek(token_1.TokenType.RBRACE)) {
            return null;
        }
        return hash;
    }
}
exports.default = Parser;
//# sourceMappingURL=parser.js.map